@@ -446,8 +446,8 @@ int main(int argc, char **argv)
 	}
 	else if (kill(pid, SIGTERM) < 0)
 	{
-	    fprintf(stderr,GT_("fetchmail: error killing %s fetchmail at %d; bailing out.\n"),
-		    bkgd ? GT_("background") : GT_("foreground"), pid);
+	    fprintf(stderr,GT_("fetchmail: error killing %s fetchmail at %ld; bailing out.\n"),
+		    bkgd ? GT_("background") : GT_("foreground"), (long)pid);
 	    exit(PS_EXCLUDE);
 	}
 	else
@@ -455,8 +455,8 @@ int main(int argc, char **argv)
 	    int maxwait;
 
 	    if (outlevel > O_SILENT)
-		fprintf(stderr,GT_("fetchmail: %s fetchmail at %d killed.\n"),
-			bkgd ? GT_("background") : GT_("foreground"), pid);
+		fprintf(stderr,GT_("fetchmail: %s fetchmail at %ld killed.\n"),
+			bkgd ? GT_("background") : GT_("foreground"), (long)pid);
 	    /* We used to nuke the other process's lock here, with
 	     * fm_lock_release(), which is broken. The other process
 	     * needs to clear its lock by itself. */
@@ -484,15 +484,15 @@ int main(int argc, char **argv)
 	else if (!implicitmode)
 	{
 	    fprintf(stderr,
-		 GT_("fetchmail: can't poll specified hosts with another fetchmail running at %d.\n"),
-		 pid);
+		 GT_("fetchmail: can't poll specified hosts with another fetchmail running at %ld.\n"),
+		 (long)pid);
 		return(PS_EXCLUDE);
 	}
 	else if (!bkgd)
 	{
 	    fprintf(stderr,
-		 GT_("fetchmail: another foreground fetchmail is running at %d.\n"),
-		 pid);
+		 GT_("fetchmail: another foreground fetchmail is running at %ld.\n"),
+		 (long)pid);
 		return(PS_EXCLUDE);
 	}
 	else if (getpid() == pid)
@@ -507,8 +507,8 @@ int main(int argc, char **argv)
 	else if (kill(pid, SIGUSR1) == 0)
 	{
 	    fprintf(stderr,
-		    GT_("fetchmail: background fetchmail at %d awakened.\n"),
-		    pid);
+		    GT_("fetchmail: background fetchmail at %ld awakened.\n"),
+		    (long)pid);
 	    return(0);
 	}
 	else
@@ -519,8 +519,8 @@ int main(int argc, char **argv)
 	     * SIGUSR1/SIGHUP transmission.
 	     */
 	    fprintf(stderr,
-		    GT_("fetchmail: elder sibling at %d died mysteriously.\n"),
-		    pid);
+		    GT_("fetchmail: elder sibling at %ld died mysteriously.\n"),
+		    (long)pid);
 	    return(PS_UNDEFINED);
 	}
     }
@@ -981,6 +981,7 @@ static void optmerge(struct query *h2, struct query *h1, int force)
     FLAG_MERGE(sslcert);
     FLAG_MERGE(sslproto);
     FLAG_MERGE(sslcertck);
+    FLAG_MERGE(sslcertfile);
     FLAG_MERGE(sslcertpath);
     FLAG_MERGE(sslcommonname);
     FLAG_MERGE(sslfingerprint);
@@ -1094,12 +1095,18 @@ static int load_params(int argc, char **argv, int optind)
 
     /*
      * If there's a defaults record, merge it and lose it.
+     * FIXME: we don't currently free all entries that might be in struct query.
      */ 
     if (querylist && strcmp(querylist->server.pollname, "defaults") == 0)
     {
+	struct query *tmpq;
+
 	for (ctl = querylist->next; ctl; ctl = ctl->next)
 	    optmerge(ctl, querylist, FALSE);
+	tmpq = querylist;
 	querylist = querylist->next;
+	free(tmpq->server.pollname);
+	free(tmpq);
     }
 
     /* don't allow a defaults record after the first */
@@ -1661,9 +1668,11 @@ static void dump_params (struct runctl *runp,
 	    printf(GT_("  SSL protocol: %s.\n"), ctl->sslproto);
 	if (ctl->sslcertck) {
 	    printf(GT_("  SSL server certificate checking enabled.\n"));
-	    if (ctl->sslcertpath != NULL)
-		printf(GT_("  SSL trusted certificate directory: %s\n"), ctl->sslcertpath);
 	}
+	if (ctl->sslcertfile != NULL)
+		printf(GT_("  SSL trusted certificate file: %s\n"), ctl->sslcertfile);
+	if (ctl->sslcertpath != NULL)
+		printf(GT_("  SSL trusted certificate directory: %s\n"), ctl->sslcertpath);
 	if (ctl->sslcommonname != NULL)
 		printf(GT_("  SSL server CommonName: %s\n"), ctl->sslcommonname);
 	if (ctl->sslfingerprint != NULL)
@@ -1891,17 +1900,14 @@ static void dump_params (struct runctl *runp,
 
 			if (ctl->server.akalist)
 			{
-			    struct idlist *idp;
-
 			    printf(GT_("  Predeclared mailserver aliases:"));
 			    for (idp = ctl->server.akalist; idp; idp = idp->next)
 				printf(" %s", idp->id);
 			    putchar('\n');
 			}
+
 			if (ctl->server.localdomains)
 			{
-			    struct idlist *idp;
-
 			    printf(GT_("  Local domains:"));
 			    for (idp = ctl->server.localdomains; idp; idp = idp->next)
 				printf(" %s", idp->id);
