@@ -187,26 +187,35 @@ struct runctl
     flag	showdots;
 };
 
+/** \name idlist */
+/** Dual-use entry of singly-linked list for storing id/status or id/id2
+ * pairs. */
 struct idlist
 {
-    char *id;
+    char *id;	/**< key */
     union
     {
 	struct
 	{
 	    int		num;
-	    flag	mark;		/* UID-index information */
-#define UID_UNSEEN	0		/* hasn't been seen */
-#define UID_SEEN	1		/* seen, but not deleted */
-#define UID_DELETED	2		/* this message has been marked deleted */
-#define UID_EXPUNGED	3		/* this message has been expunged */
+	    flag	mark;		/**< UID-index information */
         }
-	status;
-	char *id2;
-    } val;
-    struct idlist *next;
+	status;				/**< value for id/status pairs */
+	char *id2;			/**< value for id/id2 pairs */
+    } val;				/**< union to store value for key \a id */
+    struct idlist *next;		/**< pointer to next list element */
 };
 
+/** List of possible values for idlist::mark */
+enum {
+UID_UNSEEN=	0,		/**< id hasn't been seen */
+UID_SEEN=	1,		/**< id was seen, but not deleted */
+UID_DELETED=	2,		/**< this message has been marked deleted */
+UID_EXPUNGED=	3		/**< this message has been expunged */
+};
+/*@}*/
+
+
 struct query;
 
 struct method		/* describe methods for protocol state machine */
@@ -358,6 +367,7 @@ struct query
     char *sslcert;		/* optional SSL certificate file */
     char *sslproto;		/** force transport protocol (ssl2|ssl3|ssl23|tls1) - if NULL,
 				  use ssl23 for SSL and opportunistic tls1 for non-SSL connections. */
+    char *sslcertfile;		/* Trusted certificate file for checking the server cert */
     char *sslcertpath;		/* Trusted certificate directory for checking the server cert */
     flag sslcertck;		/* Strictly check the server cert. */
     char *sslcommonname;	/* CommonName to expect from server */
@@ -581,29 +591,29 @@ char *nxtaddr(const char *);
 
 /* uid.c: UID support */
 extern int dofastuidl;
-
-void initialize_saved_lists(struct query *, const char *);
-struct idlist *save_str(struct idlist **, const char *, flag);
-void free_str_list(struct idlist **);
-struct idlist *copy_str_list(struct idlist *idl);
-void save_str_pair(struct idlist **, const char *, const char *);
-void free_str_pair_list(struct idlist **);
-int delete_str(struct idlist **, long);
-struct idlist *str_in_list(struct idlist **, const char *, const flag);
-int str_nr_in_list(struct idlist **, const char *);
-int str_nr_last_in_list(struct idlist **, const char *);
-void str_set_mark( struct idlist **, const char *, const flag);
-int count_list( struct idlist **idl );
-char *str_from_nr_list( struct idlist **idl, long number );
-char *str_find(struct idlist **, long);
-struct idlist *id_find(struct idlist **idl, long);
-char *idpair_find(struct idlist **, const char *);
-void append_str_list(struct idlist **, struct idlist **);
-void expunge_uids(struct query *);
-void uid_swap_lists(struct query *);
+void initialize_saved_lists(struct query *hostlist, const char *idfile);
+void expunge_uids(struct query *ctl);
+void uid_swap_lists(struct query *ctl);
 void uid_discard_new_list(struct query *ctl);
 void uid_reset_num(struct query *ctl);
-void write_saved_lists(struct query *, const char *);
+void write_saved_lists(struct query *hostlist, const char *idfile);
+
+/* idlist.c */
+struct idlist *save_str(struct idlist **idl, const char *str, flag status);
+void free_str_list(struct idlist **idl);
+void save_str_pair(struct idlist **idl, const char *str1, const char *str2);
+struct idlist *str_in_list(struct idlist **idl, const char *str, const flag caseblind);
+int str_nr_in_list(struct idlist **idl, const char *str);
+int str_nr_last_in_list(struct idlist **idl, const char *str);
+void str_set_mark(struct idlist **idl, const char *str, const flag val);
+int count_list(struct idlist **idl);
+char *str_from_nr_list(struct idlist **idl, long number);
+char *str_find(struct idlist **idl, long number);
+struct idlist *id_find(struct idlist **idl, long number);
+char *idpair_find(struct idlist **idl, const char *id);
+int delete_str(struct idlist **idl, long num);
+struct idlist *copy_str_list(struct idlist *idl);
+void append_str_list(struct idlist **idl, struct idlist **nidl);
 
 /* rcfile_y.y */
 int prc_parse_file(const char *, const flag);
@@ -651,7 +661,7 @@ extern char currentwd[1024], rcfiledir[1024];
 struct query *hostalloc(struct query *); 
 int parsecmdline (int, char **, struct runctl *, struct query *);
 char *prependdir (const char *, const char *);
-unsigned char *MD5Digest (unsigned const char *);
+char *MD5Digest (unsigned const char *);
 void hmac_md5 (const unsigned char *, size_t, const unsigned char *, size_t, unsigned char *, size_t);
 int POP3_auth_rpa(char *, char *, int socket);
 typedef RETSIGTYPE (*SIGHANDLERTYPE) (int);
