@@ -491,7 +491,7 @@ static int pop3_getauth(int sock, struct query *ctl, char *greeting)
 		* (see below). */
 	       if (gen_transact(sock, "STLS") == PS_SUCCESS
 		       && SSLOpen(sock, ctl->sslcert, ctl->sslkey, "tls1", ctl->sslcertck,
-			   ctl->sslcertpath, ctl->sslfingerprint, commonname,
+			   ctl->sslcertfile, ctl->sslcertpath, ctl->sslfingerprint, commonname,
 			   ctl->server.pollname, &ctl->remotename) != -1)
 	       {
 		   /*
@@ -690,10 +690,10 @@ static int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	msg = (char *)xmalloc((end-start+1) + strlen(ctl->password) + 1);
 	strcpy(msg,start);
 	strcat(msg,ctl->password);
-	strcpy((char *)ctl->digest, (char *)MD5Digest((unsigned char *)msg));
+	strcpy((char *)ctl->digest, MD5Digest((unsigned char *)msg));
 	free(msg);
 
-	ok = gen_transact(sock, "APOP %s %s", ctl->remotename, ctl->digest);
+	ok = gen_transact(sock, "APOP %s %s", ctl->remotename, (char *)ctl->digest);
 	break;
 
     case P_RPOP:
@@ -856,7 +856,7 @@ static int pop3_fastuidl( int sock,  struct query *ctl, unsigned int count, int
 	    if (mark == UID_DELETED || mark == UID_EXPUNGED)
 	    {
 		if (outlevel >= O_VERBOSE)
-		    report(stderr, GT_("id=%s (num=%d) was deleted, but is still present!\n"), id, try_nr);
+		    report(stderr, GT_("id=%s (num=%u) was deleted, but is still present!\n"), id, try_nr);
 		/* just mark it as seen now! */
 		newl->val.status.mark = mark = UID_SEEN;
 	    }
@@ -1009,9 +1009,13 @@ static int pop3_getrange(int sock,
     /* get the total message count */
     gen_send(sock, "STAT");
     ok = pop3_ok(sock, buf);
-    if (ok == 0)
-	sscanf(buf,"%d %d", countp, bytes);
-    else
+    if (ok == 0) {
+	int asgn;
+
+	asgn = sscanf(buf,"%d %d", countp, bytes);
+	if (asgn != 2)
+		return PS_PROTOCOL;
+    } else
 	return(ok);
 
     /*
