@@ -21,10 +21,11 @@
 #include  <unistd.h>
 #include  <ctype.h>
 #include  <string.h>
+#include  <sys/types.h>
 
 #include  "socket.h"
 #include  "fetchmail.h"
-#include  "md5.h"
+#include  "fm_md5.h"
 #include  "i18n.h"
 
 #ifdef TESTMODE
@@ -114,7 +115,7 @@ int POP3_auth_rpa (char *userid, char *passphrase, int socket)
     char buf [POPBUFSIZE];
     char *bufp;
     int      status,aulin,kuslin;
-    char* stdec[4] = { N_("Success") ,
+    const char* stdec[4] = { N_("Success") ,
 		       N_("Restricted user (something wrong with account)") ,
 		       N_("Invalid userid or passphrase") ,
 		       N_("Deity error") };
@@ -527,7 +528,7 @@ static int DecBase64(char *bufp)
 	    else if ( ch=='+'                )   newx = 62;
 	    else if ( ch=='/'                )   newx = 63;
 	    else {
-		report(stderr,  GT_("dec64 error at char %d: %x\n"), inp - bufp, ch);
+	        report(stderr,  GT_("dec64 error at char %d: %x\n"), (int)(inp - bufp), ch);
 		return(0);
 	    }
 	    part=((part & 0x3F)*64) + newx;
@@ -883,7 +884,7 @@ static void md5(const void *in_,int len,unsigned char *out)
     MD5Final(  out, &md5context );
     if (outlevel >= O_DEBUG)
     {
-	report(stdout, GT_("MD5 result is: \n"));
+	report(stdout, GT_("MD5 result is:\n"));
 	for (i=0; i<16; i++)
 	{
 	    report_build(stdout, "%02X ",out[i]);
